pub struct InnerShota {
    array: [u8; 23],
    len: u8,
}

impl Default for InnerShota {
    fn default() -> Self {
        InnerShota {
            array: [0; 23],
            len: 0,
        }
    }
}

pub enum Shota {
    Stack(InnerShota),
    Heap(String),
}

impl Default for Shota {
    fn default() -> Self {
        Shota::Stack(InnerShota::default())
    }
}

impl Shota {
    pub fn function_name_here(&self) -> &[u8] {
        match self {
            Shota::Stack(ref _x) => unimplemented!("not implemented"),
            Shota::Heap(ref _x) => unimplemented!("not implemented"),
        }
    }

    pub fn as_bytes(&self) -> &[u8] {
        match self {
            Shota::Stack(ref x) => &x.array,
            Shota::Heap(ref x) => x.as_bytes(),
        }
    }

    pub fn as_mut_str(&mut self) -> &mut str {
        match self {
            // FIXME: Just panic for now, but we should perform error checking like
            Shota::Stack(ref mut x) => std::str::from_utf8_mut(&mut x.array).unwrap(),
            Shota::Heap(ref mut x) => x.as_mut_str(),
        }
    }

    pub unsafe fn as_mut_vec(&mut self) -> &mut Vec<u8> {
        match self {
            // FIXME: Just panic for now, but we should perform error checking like
            Shota::Stack(ref mut _x) => unimplemented!("this is a lot harder to implement...I am quite sure we have to invoke some low level contruct to be able to manipulate this stuff on the stack."),
            Shota::Heap(ref mut x) => x.as_mut_vec(),
        }
    }

    pub fn as_str(&self) -> &str {
        match self {
            // FIXME: Just panic for now, but we should perform error checking like
            Shota::Stack(ref x) => std::str::from_utf8(&x.array).unwrap(),
            Shota::Heap(ref x) => x.as_str(),
        }
    }

    pub fn capacity(&self) -> usize {
        match self {
            Shota::Stack(ref _x) => {
                std::mem::size_of::<InnerShota>() - std::mem::size_of::<usize>()
            }
            Shota::Heap(ref x) => x.len(),
        }
    }

    pub fn clear(mut self) {
        match self {
            Shota::Stack(ref mut x) => *x = InnerShota::default(),
            Shota::Heap(ref mut x) => x.clear(),
        }
    }

    pub fn len(&self) -> usize {
        match self {
            Shota::Stack(ref x) => x.len as usize,
            Shota::Heap(ref x) => x.len(),
        }
    }
}
